@@ -11,14 +11,72 @@ from pdbmender.constants import (
 SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
 
 
+def correct_insertion_codes(pdb_in, pdb_out, insertion_offset=3000):
+    with open(pdb_in) as f:
+        lines = f.readlines()
+
+    ninsertions = -1
+    inserted_block = None
+    renumbered_res = {}
+    new_lines = ""
+    for line in lines:
+        if line.startswith("ATOM "):
+            aname, anumb, resname, chain, resnumb, x, y, z = read_pdb_line(line)
+
+            insertion_code = line[26]
+            if insertion_code.strip():
+                new_res_trigger = False
+                if inserted_block != insertion_code:
+                    ninsertions += 1
+                    new_res_trigger = True
+
+                inserted_block = insertion_code
+                resnumb_offset = insertion_offset + ninsertions
+                resnumb += resnumb_offset
+
+                if new_res_trigger:
+                    while (
+                        chain in renumbered_res.keys()
+                        and resnumb in renumbered_res[chain].keys()
+                    ):
+                        resnumb += 1
+                    if resnumb not in renumbered_res.keys():
+                        if chain not in renumbered_res:
+                            renumbered_res[chain] = {}
+                        renumbered_res[chain][resnumb] = (
+                            resnumb - resnumb_offset,
+                            insertion_code,
+                        )
+            else:
+                ninsertions = -1
+
+            new_line = new_pdb_line(
+                anumb,
+                aname,
+                resname,
+                resnumb,
+                x,
+                y,
+                z,
+                chain=chain,
+            )
+            new_lines += new_line
+    with open(pdb_out, "w") as f:
+        f.write(new_lines)
+
+    return renumbered_res
+
+
 def mend_pdb(pdb_to_clean, pdb_cleaned, ff, ffout, logfile="LOG_pdb2pqr", hopt=True):
+    pdb_fixed = pdb_to_clean.replace(".pdb", "_fixed.pdb")
+    renumbered_res = correct_insertion_codes(pdb_to_clean, pdb_fixed)
     try:
         # TODO: Port pdb2pqr to py3 and import it as a module
         cmd = (
             "python2 {0}/pdb2pqr/pdb2pqr.py {1} {2} "
             "--ff {3} --ffout {4} --drop-water -v --chain {6} > {5} 2>&1 ".format(
                 SCRIPT_DIR,
-                pdb_to_clean,
+                pdb_fixed,
                 pdb_cleaned,
                 ff,
                 ffout,
@@ -39,6 +97,7 @@ def mend_pdb(pdb_to_clean, pdb_cleaned, ff, ffout, logfile="LOG_pdb2pqr", hopt=T
                 e.stderr.decode("ascii")
             )
         )
+    return renumbered_res
 
 
 def add_tautomers(
@@ -51,7 +110,7 @@ def add_tautomers(
     to_exclude=[],
     terminal_offset=5000,
 ):
-    removed_pdb, resnumbered_res = prepare_for_addHtaut(
+    removed_pdb = prepare_for_addHtaut(
         pdb_in, pdb_prep, chains_res, to_exclude, terminal_offset=terminal_offset
     )
     sites_addHtaut = select_tautomer_sites(chains_res)
@@ -81,7 +140,7 @@ def add_tautomers(
                     e.stderr.decode("ascii")
                 )
             )
-    return removed_pdb, resnumbered_res
+    return removed_pdb
 
 
 def correct_names(resnumb, resname, aname, titrating_sites, termini):
@@ -119,9 +178,7 @@ def correct_names(resnumb, resname, aname, titrating_sites, termini):
     return aname, resname
 
 
-def prepare_for_addHtaut(
-    pdb_in, pdb_out, chains_res, to_exclude, terminal_offset=5000, insertion_offset=3000
-):
+def prepare_for_addHtaut(pdb_in, pdb_out, chains_res, to_exclude, terminal_offset=5000):
     with open(pdb_in) as f:
         content = f.readlines()
 
@@ -144,9 +201,6 @@ def prepare_for_addHtaut(
     removed_pdb_lines = []
     resnumb_max = 0
     chains = sites.keys()
-    ninsertions = -1
-    inserted_block = None
-    renumbered_res = {}
     for line in content:
         if line.startswith("ATOM"):
             termini_trigger = False
@@ -178,26 +232,6 @@ def prepare_for_addHtaut(
                 else:
                     continue
 
-            insertion_code = line[26]
-            if insertion_code.strip():
-                new_res_trigger = False
-                if inserted_block != insertion_code:
-                    ninsertions += 1
-                    new_res_trigger = True
-
-                inserted_block = insertion_code
-                resnumb_offset = insertion_offset + ninsertions
-                resnumb += resnumb_offset
-
-                if new_res_trigger:
-                    while resnumb in renumbered_res.keys():
-                        resnumb += 1
-                    if resnumb not in renumbered_res.keys():
-                        renumbered_res[resnumb] = (
-                            resnumb - resnumb_offset,
-                            insertion_code,
-                        )
-
             new_line = new_pdb_line(
                 anumb,
                 aname,
@@ -234,7 +268,7 @@ def prepare_for_addHtaut(
 
     with open("removed.pqr", "w") as f_new:
         f_new.write(removed_pdb_text)
-    return removed_pdb_text, renumbered_res
+    return removed_pdb_text
 
 
 def get_cys_bridges(f_pdb2pqr_log):
@@ -314,6 +348,7 @@ def identify_tit_sites(f_in, chains, nomenclature="PDB", add_ser_thr=False):
             (aname, anumb, resname, chain, resnumb, x, y, z) = read_pdb_line(line)
             insertion_code = line[26].strip()
             if insertion_code:
+                print("NOT CONSIDERING:", line)
                 continue
 
             last_res = resnumb
